/* dma.c: DMA initializer
 *
 * 2001 Camilo Alejandro Arboleda
 *
 */


#define DMA_MEMBASE    0x68000u  /* This memory was freed by GRUB after booting
                                  * booting RTEMS image
                                  */
#define DMA_BUFSIZE    0x10000u


#include <bsp.h>
#include <irq.h>
#include <rtems/libio.h>
/* #include <rtems/libio_.h> */
#include <assert.h>
#include "dma.h"
#include "floppy.h"

static rtems_id dma_buff_id;
static int dma_open_count=0;

/*===================================================================
 * Driver entry points
 */

/*-------------------------------------------------------------------
 * dma_initialize(major, minor, arg)
 */
rtems_device_driver
dma_initialize ( major,minor,arg )
                rtems_device_major_number major;
                rtems_device_minor_number minor;
		void                      *arg;
{
   rtems_status_code status;
   void *base = DMA_MEMBASE;

   outport_byte(DMA_INIT, DMA_RESET_VAL);    /* reset the dma controller */
   outport_byte(DMA_FLIPFLOP, 0);	     /* write anything to reset it */

   status = rtems_partition_create(rtems_build_name('P','A','R','d'),
                                   base, DMA_BUFSIZE, RTEMS_LOCAL,
                                   &dma_buf_id);
   if (status != RTEMS_SUCCESSFUL) return status;
   printk("Dma Initialized\n");
   return RTEMS_SUCCESSFUL;
};

/*-------------------------------------------------------------------
 * dma_open(major, minor, arg)
 */
rtems_device_driver
dma_open       ( major,minor,arg )
                rtems_device_major_number major;
                rtems_device_minor_number minor;
                void                      *arg;

{
   dma_open_count++;
   return RTEMS_SUCCESSFUL;
};

/*-------------------------------------------------------------------
 * dma_close(major, minor, arg)
 */
rtems_device_driver
dma_close      ( major,minor,arg )
                rtems_device_major_number major;
                rtems_device_minor_number minor;
                void                      *arg;
{
   if ( dma_open_count-- <= 0 ) dma_open_count = 0;
   return RTEMS_SUCCESSFUL;
};

/*-------------------------------------------------------------------
 * dma_read(major, minor, arg)
 * Read the floppy device.
 */
rtems_device_driver
dma_read       ( major,minor,arg )
                rtems_device_major_number major;
                rtems_device_minor_number minor;
                void                      *arg;
{
   return RTEMS_SUCCESSFUL;
};

/*-------------------------------------------------------------------
 * dma_write(major, minor, arg)
 */
rtems_device_driver
dma_write      ( major,minor,arg )
                rtems_device_major_number major;
                rtems_device_minor_number minor;
                void                      *arg;
{
   return RTEMS_SUCCESSFUL;
};

/*-------------------------------------------------------------------
 * dma_control(major, minor, arg)
 */
rtems_device_driver
dma_control    ( major,minor,arg )
               rtems_device_major_number major;
               rtems_device_minor_number minor;
               void                      *arg;
{
   return RTEMS_SUCCESSFUL;
};
